#ifndef MAILSTORAGE_TYPES_H

#define MAILSTORAGE_TYPES_H

#ifdef __cplusplus
extern "C" {
#endif

struct mail_storage;

typedef struct mail_storage_driver mail_storage_driver;


/*
  There is three kinds of identities :
  - storage
  - folders
  - session

  A storage (struct mail_storage) represents whether a server or
  a main path,

  A storage can be an IMAP server, the root path of a MH or a mbox file.

  Folders (struct mail_folder) are the mailboxes we can
  choose in the server or as sub-folder of the main path.

  Folders for IMAP are the IMAP mailboxes, for MH this is one of the
  folder of the MH storage, for mbox, there is only one folder, the
  mbox file content;

  A mail session (struct mailsession) is whether a connection to a server
  or a path that is open. It is the abstraction lower folders and storage.
  It allow us to send commands.

  We have a session driver for mail session for each kind of storage.

  From a session, we can get a message (struct mailmessage) to read.
  We have a message driver for each kind of storage.
*/

/*
  mail_storage_driver is the driver structure for mail storages

  - name is the name of the driver
  
  - connect() connects the storage to the remote access or to
      the path in the local filesystem.
      
  - get_folder() can have two kinds of behaviour.
      Either it creates a new session and independant from the session
      used by the storage and select the given mailbox or
      it selects the given mailbox in the current session.
      It depends on the efficiency of the mail driver.

  - free_data() frees the data created with mail_storage constructor.
*/

struct mail_storage_driver {
  char * name;
  int (* connect)(struct mail_storage * storage);
  int (* get_folder)(struct mail_storage * storage,
      char * pathname, mailsession ** result);
  void (* free_data)(void * data);
};

/*
  mail_storage is the data structure for a storage

  - id is the name of the storage, it can be NULL.
  
  - data is the data specific to the driver.
      This is the internal state of the storage.

  - session is the session related to the storage.

  - driver is the driver for the storage.

  - shared_folders is the list of folders returned by the storage.
*/

struct mail_storage {
  char * id;
  void * data;
  mailsession * session;
  mail_storage_driver * driver;
  clist * shared_folders; /* list of (struct mail_folder *) */
};



/*
  mail_folder is the data structure for a mailbox

  - pathname is the path of the mailbox on the storage

  - virtual_name is the folder identifier, it can be a path,
      a name or NULL.

  - storage is the storage to which the folder belongs to.

  - session is the session related to the folder. It can be
      different of the session of the storage.

  - shared_session is != 0 if the session is the same as the
      session of the storage.

  - pos is the position of the folder in the "shared_folders" field
      of the storage.

  folders can be chained into a tree.

  - parent is the parent of the folder.

  - sibling_index is the index of the folder in the list of children
      of the parent.
      
  - children is the folder.
*/

struct mail_folder {
  char * pathname;
  char * virtual_name;
  
  struct mail_storage * storage;

  mailsession * session;
  int shared_session;
  clistiter * pos;

  struct mail_folder * parent;
  int32_t sibling_index;
  carray * children; /* array of (struct mail_folder *) */
};

#ifdef __cplusplus
}
#endif

#endif
